@@ -1,12 +1,35 @@
-# %%
-import numpy as np
-import df2onehot
-print(df2onehot.__version__)
+from df2onehot import df2onehot
+import pandas as pd
+import pypickle
+filepath = 'D://GITLAB/PROJECTS/shodanrws/results/shodandata.pkl'
+X = pypickle.load(filepath)
+df = pd.json_normalize(X['matches'])
+
+Xout = df2onehot(df, deep_extract=True, verbose=3)
+# Xext = df2onehot(df, deep_extract=deep_extract, y_min=1, perc_min_num=0.8, verbose=3)
+
+Xout['onehot'].shape
+Xout['onehot'].sum(axis=0)
+len(Xout['labx'])
+
+Xout['numeric'].shape
+len(Xout['labels'])
+
+Xout['df'].shape
+
+import sweetviz as sv
+my_report = sv.analyze(Xout['df'].iloc[:,0:100], pairwise_analysis='off')
+my_report.show_html() # Default arguments will generate to "SWEETVIZ_REPORT.html"
+
+# Doe iets met timestamp
+# Doe iets met locatie
 
 
 # %%
 import numpy as np
 import df2onehot
+# print(df2onehot.__version__)
+
 df = df2onehot.import_example()
 colnames = np.array(['3','4','5','6','7','8','9','11','10','1'])
 df['lists'] = np.nan
@@ -22,7 +45,7 @@ df['list2'].iloc[890] = 1
 df['list2'].iloc[888] = 10
 
 # Run df2onehot
-out = df2onehot.df2onehot(df)
+out = df2onehot.df2onehot(df, deep_extract=True)
 
 # # TEST 1: check output is unchanged
 # out = df2onehot(df)
@@ -36,7 +59,7 @@ df = df2onehot.import_example()
 
 # %% Convert
 out = df2onehot.df2onehot(df)
-
+out['numeric']
 
 # %% Force feature (int or float) to be numeric if unique non-zero values are above percentage.
 out = df2onehot.df2onehot(df, perc_min_num=0.8)
